@@ -10,6 +10,27 @@ import (
 	"github.com/yuin/goldmark/util"
 )
 
+var uncloseCounterKey = parser.NewContextKey()
+
+type unclosedCounter struct {
+	Single int
+	Double int
+}
+
+func (u *unclosedCounter) Reset() {
+	u.Single = 0
+	u.Double = 0
+}
+
+func getUnclosedCounter(pc parser.Context) *unclosedCounter {
+	v := pc.Get(uncloseCounterKey)
+	if v == nil {
+		v = &unclosedCounter{}
+		pc.Set(uncloseCounterKey, v)
+	}
+	return v.(*unclosedCounter)
+}
+
 // TypographicPunctuation is a key of the punctuations that can be replaced with
 // typographic entities.
 type TypographicPunctuation int
@@ -139,7 +160,7 @@ func NewTypographerParser(opts ...TypographerOption) parser.InlineParser {
 }
 
 func (s *typographerParser) Trigger() []byte {
-	return []byte{'\'', '"', '-', '.', '<', '>'}
+	return []byte{'\'', '"', '-', '.', ',', '<', '>', '*', '['}
 }
 
 func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser.Context) gast.Node {
@@ -193,6 +214,7 @@ func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser
 		if d == nil {
 			return nil
 		}
+		counter := getUnclosedCounter(pc)
 		if c == '\'' {
 			if s.Substitutions[Apostrophe] != nil {
 				// Handle decade abbrevations such as '90s
@@ -201,13 +223,20 @@ func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser
 					if len(line) > 4 {
 						after = util.ToRune(line, 4)
 					}
-					if len(line) == 3 || unicode.IsSpace(after) || unicode.IsPunct(after) {
+					if len(line) == 3 || util.IsSpaceRune(after) || util.IsPunctRune(after) {
 						node := gast.NewString(s.Substitutions[Apostrophe])
 						node.SetCode(true)
 						block.Advance(1)
 						return node
 					}
 				}
+				// special cases: 'twas, 'em, 'net
+				if len(line) > 1 && (unicode.IsPunct(before) || unicode.IsSpace(before)) && (line[1] == 't' || line[1] == 'e' || line[1] == 'n' || line[1] == 'l') {
+					node := gast.NewString(s.Substitutions[Apostrophe])
+					node.SetCode(true)
+					block.Advance(1)
+					return node
+				}
 				// Convert normal apostrophes. This is probably more flexible than necessary but
 				// converts any apostrophe in between two alphanumerics.
 				if len(line) > 1 && (unicode.IsDigit(before) || unicode.IsLetter(before)) && (unicode.IsLetter(util.ToRune(line, 1))) {
@@ -219,7 +248,7 @@ func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser
 			}
 			if s.Substitutions[LeftSingleQuote] != nil && d.CanOpen && !d.CanClose {
 				nt := LeftSingleQuote
-				// special cases: Alice's, I'm ,Don't, You'd
+				// special cases: Alice's, I'm, Don't, You'd
 				if len(line) > 1 && (line[1] == 's' || line[1] == 'm' || line[1] == 't' || line[1] == 'd') && (len(line) < 3 || util.IsPunct(line[2]) || util.IsSpace(line[2])) {
 					nt = RightSingleQuote
 				}
@@ -227,6 +256,9 @@ func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser
 				if len(line) > 2 && ((line[1] == 'v' && line[2] == 'e') || (line[1] == 'l' && line[2] == 'l') || (line[1] == 'r' && line[2] == 'e')) && (len(line) < 4 || util.IsPunct(line[3]) || util.IsSpace(line[3])) {
 					nt = RightSingleQuote
 				}
+				if nt == LeftSingleQuote {
+					counter.Single++
+				}
 
 				node := gast.NewString(s.Substitutions[nt])
 				node.SetCode(true)
@@ -234,36 +266,46 @@ func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser
 				return node
 			}
 			if s.Substitutions[RightSingleQuote] != nil {
-				isClose := d.CanClose && !d.CanOpen
-				maybeClose := d.CanClose && d.CanOpen && len(line) > 1 && (line[1] == ',' || line[1] == '.' || line[1] == '!' || line[1] == '?') && (len(line) == 2 || (len(line) > 2 && util.IsPunct(line[2]) || util.IsSpace(line[2])))
-				if isClose || maybeClose {
+				// plural possesives and abbreviations: Smiths', doin'
+				if len(line) > 1 && unicode.IsSpace(util.ToRune(line, 0)) || unicode.IsPunct(util.ToRune(line, 0)) && (len(line) > 2 && !unicode.IsDigit(util.ToRune(line, 1))) {
 					node := gast.NewString(s.Substitutions[RightSingleQuote])
 					node.SetCode(true)
 					block.Advance(1)
 					return node
 				}
 			}
+			if s.Substitutions[RightSingleQuote] != nil && counter.Single > 0 {
+				isClose := d.CanClose && !d.CanOpen
+				maybeClose := d.CanClose && d.CanOpen && len(line) > 1 && unicode.IsPunct(util.ToRune(line, 1)) && (len(line) == 2 || (len(line) > 2 && util.IsPunct(line[2]) || util.IsSpace(line[2])))
+				if isClose || maybeClose {
+					node := gast.NewString(s.Substitutions[RightSingleQuote])
+					node.SetCode(true)
+					block.Advance(1)
+					counter.Single--
+					return node
+				}
+			}
 		}
 		if c == '"' {
-			// special case: 7'1""
-			if len(line) > 1 && line[1] == '"' && unicode.IsDigit(before) {
-				node := gast.NewString(line[0:2])
-				block.Advance(2)
-				return node
-			}
 			if s.Substitutions[LeftDoubleQuote] != nil && d.CanOpen && !d.CanClose {
 				node := gast.NewString(s.Substitutions[LeftDoubleQuote])
 				node.SetCode(true)
 				block.Advance(1)
+				counter.Double++
 				return node
 			}
-			if s.Substitutions[RightDoubleQuote] != nil {
+			if s.Substitutions[RightDoubleQuote] != nil && counter.Double > 0 {
 				isClose := d.CanClose && !d.CanOpen
-				maybeClose := d.CanClose && d.CanOpen && len(line) > 1 && (line[1] == ',' || line[1] == '.' || line[1] == '!' || line[1] == '?') && (len(line) == 2 || (len(line) > 2 && util.IsPunct(line[2]) || util.IsSpace(line[2])))
+				maybeClose := d.CanClose && d.CanOpen && len(line) > 1 && (unicode.IsPunct(util.ToRune(line, 1))) && (len(line) == 2 || (len(line) > 2 && util.IsPunct(line[2]) || util.IsSpace(line[2])))
 				if isClose || maybeClose {
+					// special case: "Monitor 21""
+					if len(line) > 1 && line[1] == '"' && unicode.IsDigit(before) {
+						return nil
+					}
 					node := gast.NewString(s.Substitutions[RightDoubleQuote])
 					node.SetCode(true)
 					block.Advance(1)
+					counter.Double--
 					return node
 				}
 			}
@@ -273,7 +315,7 @@ func (s *typographerParser) Parse(parent gast.Node, block text.Reader, pc parser
 }
 
 func (s *typographerParser) CloseBlock(parent gast.Node, pc parser.Context) {
-	// nothing to do
+	getUnclosedCounter(pc).Reset()
 }
 
 type typographer struct {
